/*  LAST EDIT: Wed Dec 18 00:34:54 1996 by Adabas D Database (gutamil.v12-berlin.de!adabas)  */
#ifdef __cplusplus
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "tcl.h"
#include "../../extension/adabas.h"
#include "../../extension/AdabasApi.h"
#ifdef __cplusplus
}
#endif

/* This may be defined in the Makefile. */
#ifndef TCLLIB
#  define TCLLIB "libtcl8.0"
#endif

#ifdef HAS_TCL_OBJECTS
static int dummyLength;
#define ResString Tcl_Obj *
#define ResStringInit(obj) ((obj) = Tcl_NewObj ())
#define ResStringFree(obj) (Tcl_SetStringObj (obj, "", 1))
#define ResStringValue(interp,obj) \
     (Tcl_GetStringFromObj ((interp),(obj), &dummyLength))
#define ResStringGetResult(interp,obj) \
     ((obj) = Tcl_GetObjResult (interp), Tcl_IncrRefCount (obj))

#define GetIntFromResult(errInterp,resInterp,longPtr) \
     (Tcl_GetIntFromObj ((errInterp), Tcl_GetObjResult (resInterp), (longPtr)))

#define GetListFromResult(interp,objc,objv) \
    (Tcl_ListObjGetElements (interp, Tcl_GetObjResult (interp), (objc), (objv)))
#define FreeListObjv(objv)
#else
#define objc    argc
#define objv    argv
#define Tcl_Obj char

#define ResString Tcl_DString

#define Tcl_GetStringFromObj(interp,obj,lenPtr) \
	((*(lenPtr) = strlen(obj)), (obj))

#define ResStringInit(obj) (Tcl_DStringInit(&(obj))
#define ResStringFree(obj) (Tcl_DStringFree(&(obj))
#define ResStringValue(interp,obj) (Tcl_DStringValue(&(obj))
#define ResStringGetResult(interp,obj) (Tcl_DStringGetResult ((interp),&(obj)))

#define GetIntFromResult(errInterp,resInterp,longPtr) \
     (Tcl_GetInt ((errInterp), (resInterp)->result, (longPtr)))

#define GetListFromResult(interp,objc,objv) \
    (Tcl_SplitList (interp, (interp)->result, (objc), (objv)))
#define FreeListObjv(objv) ckfree(objv)
#endif

/*
 * And here are the global variables of the extension.
 */

Tcl_Interp *AdabasTclInterp;
ResString   AdabasErrorMsg;
long        AdabasRC       = 0;
long        AdabasErrorPos = 0;

MODULE = Adabas		PACKAGE = Adabas

PROTOTYPES: ENABLE

BOOT:
{
  /*
   * First create the Tcl interpreter, where all the Adabas stuff will
   * be evaluated.
   */
  AdabasTclInterp = Tcl_CreateInterp();

  /*
   * The following lines seem funny, since I would expect, that the Tcl_*
   * procedures are already present, as I just invoked Tcl_CreateInterp.
   * But these procedures may be loaded statically (and already resolved)
   * into Adabas.so and thus not applicable. And if they are already loaded,
   * the following command does almost nothing.
   * Don't ask the returncode here, since it may be perfectly alright to fail,
   * if tcl is loaded statically and there is no shared library available.
   */
  Tcl_VarEval (AdabasTclInterp, "load ", TCLLIB, "[info sharedlibextension]",
	       (char *) NULL);

  /*
   * adasql.tcl asks about it, so we better define it here.
   * We set it to false, since all the sqlsh commands are not used here.
   */
  Tcl_SetVar (AdabasTclInterp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);

  /*
   * Now load the AdabasTcl extension (where all this is about).
   */
  /* if (Tcl_Eval (AdabasTclInterp,
		"load $env(DBROOT)/lib/Adabastcl[info sharedlibextension]"
		) != TCL_OK)*/
  if (Adabastcl_Init (AdabasTclInterp) != TCL_OK)
    fprintf (stderr, "%s\n", AdabasTclInterp->result);

  /*
   * Initialize the global variables.
   */
  ResStringInit (AdabasErrorMsg);
}

char *
version ()
CODE:
{
  ResStringFree (AdabasErrorMsg);
  Tcl_Eval (AdabasTclInterp, "adabas version");
  RETVAL = AdabasTclInterp->result;
}
OUTPUT:
RETVAL

char *
errortxt ()
CODE:
{
  RETVAL = ResStringValue (AdabasTclInterp, AdabasErrorMsg);
}
OUTPUT:
RETVAL

int
errorpos ()
CODE:
{
  RETVAL = AdabasErrorPos;
}
OUTPUT:
RETVAL

int
rc ()
CODE:
{
  RETVAL = AdabasRC;
}
OUTPUT:
RETVAL

AdabasInfo *
logon (connectString, serverdb="", sqlMode="adabas")
	char *connectString
	char *serverdb;
	char *sqlMode
CODE:
{
  int         isolation   = -1;
  service     serv        = SQL_USER;
  int         controlAppl = 0;
  AdabasInfo *logonHandle;

  ResStringFree (AdabasErrorMsg);

  logonHandle = AdabasLogon (AdabasTclInterp, serverdb, connectString,
			     isolation, sqlMode, serv, controlAppl);

  if (!logonHandle)
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);

  RETVAL = logonHandle;
}
OUTPUT:
RETVAL

void
logoff (logonHandle)
	AdabasInfo *logonHandle
CODE:
{
  ResStringFree (AdabasErrorMsg);
  AdabasLogoff (AdabasTclInterp, logonHandle);
}

CursorInfo *
open (logonHandle)
	AdabasInfo *logonHandle
CODE:
{
  CursorInfo *cursorHandle;

  ResStringFree (AdabasErrorMsg);
  cursorHandle = AdabasOpen (AdabasTclInterp, logonHandle);

  if (!cursorHandle)
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);

  RETVAL = cursorHandle;
}
OUTPUT:
RETVAL

void
close (cursorHandle)
	CursorInfo *cursorHandle
CODE:
{
  ResStringFree (AdabasErrorMsg);
  AdabasClose (AdabasTclInterp, cursorHandle);
}

int
sql (cursorHandle, sqlStatement, sqlMode="", resultTable="")
	CursorInfo *cursorHandle
	char       *sqlStatement
	char       *sqlMode
	char       *resultTable
CODE:
{
  int messType       = M_DBS;
  int withParameters = 0;

  if (AdabasSql (AdabasTclInterp,  cursorHandle, messType, withParameters,
		 sqlStatement, resultTable, sqlMode) == TCL_OK) {
    ResStringFree (AdabasErrorMsg);
    AdabasRC       = 0;
    AdabasErrorPos = 0;
  } else {
    if (GetIntFromResult ((Tcl_Interp *) NULL, AdabasTclInterp, &AdabasRC)
	!= TCL_OK) {
      ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
      AdabasRC = -1;
    } else if (Tcl_Eval (AdabasTclInterp, "set adamsg(errortxt)") == TCL_OK)
      ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
    else 
      ResStringFree (AdabasErrorMsg);
    if (Tcl_Eval (AdabasTclInterp, "set adamsg(errorpos)") != TCL_OK
	|| GetIntFromResult (AdabasTclInterp, AdabasTclInterp,
			     &AdabasErrorPos) != TCL_OK)
      AdabasErrorPos = 1;
  }
  RETVAL = AdabasRC;
}
OUTPUT:
RETVAL

void
fetch (cursorHandle, position="", sqlMode="")
	CursorInfo *cursorHandle
	char       *position
	char       *sqlMode
PPCODE:
{
  int       objc;
  Tcl_Obj **objv;
  Tcl_Obj **currObj;
  char     *currResult;
  int       count    = 1;
  int       arraySet = 0;
  int       length;

  if (AdabasFetch (AdabasTclInterp, cursorHandle, position,
		   count, sqlMode, arraySet) != TCL_OK) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
  } else if (GetListFromResult (AdabasTclInterp, &objc, &objv) == TCL_ERROR) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
    EXTEND (sp, 1);
    PUSHs (sv_newmortal ());
  } else {
    ResStringFree (AdabasErrorMsg);
    EXTEND (sp, objc);
    currObj = objv;
    while (objc--) {
      currResult = Tcl_GetStringFromObj (AdabasTclInterp, *currObj, &length);
      PUSHs (sv_2mortal (newSVpv (currResult, length)));
      currObj++;
    }
    FreeListObjv (objv);
  }
}

void
fetchHash (cursorHandle, position="", sqlMode="")
	CursorInfo *cursorHandle
	char       *position
	char       *sqlMode
PPCODE:
{
  int       objc;
  Tcl_Obj **objv;
  Tcl_Obj **currObj;
  char     *currResult;
  int       count    = 1;
  int       arraySet = 1;
  int       length;

  if (AdabasFetch (AdabasTclInterp, cursorHandle, position,
		   count, sqlMode, arraySet) != TCL_OK) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
  } else if (GetListFromResult (AdabasTclInterp, &objc, &objv) == TCL_ERROR) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
  } else {
    ResStringFree (AdabasErrorMsg);
    
    EXTEND (sp, objc);
    currObj = objv;
    while (objc--) {
      currResult = Tcl_GetStringFromObj (AdabasTclInterp, *currObj, &length);
      PUSHs (sv_2mortal (newSVpv (currResult, length)));
      currObj++;
    }
    FreeListObjv (objv);
  }
}

int
commit (logonHandle)
	AdabasInfo *logonHandle
CODE:
{
  ResStringFree (AdabasErrorMsg);
  if (AdabasCommit (AdabasTclInterp, logonHandle) != TCL_OK) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
    RETVAL = 1;
  } else
    RETVAL = 0;
}
OUTPUT:
RETVAL

int
rollback (logonHandle)
	AdabasInfo *logonHandle
CODE:
{
  ResStringFree (AdabasErrorMsg);
  if (AdabasRollback (AdabasTclInterp, logonHandle) != TCL_OK) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
    RETVAL = 1;
  } else
    RETVAL = 0;
}
OUTPUT:
RETVAL

int
autocommit (logonHandle, on)
	AdabasInfo *logonHandle
	int         on
CODE:
{
  ResStringFree (AdabasErrorMsg);
  if (AdabasAutoCommit (AdabasTclInterp, logonHandle, on) != TCL_OK) {
    ResStringGetResult (AdabasTclInterp, AdabasErrorMsg);
    RETVAL = 1;
  } else
    RETVAL = 0;
}
OUTPUT:
RETVAL


# ---------------------------------------------------------------------
# Some stuff for editing with emacs...
# ---------------------------------------------------------------------
# Local Variables: 
# mode: c
# mode: font-lock
# End: 



